/***************************************************************************
 *   Copyright (C) 2008 by Bram Schoenmakers <bramschoenmakers@kde.nl>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "commandwatch.h"

#include <QAction>
#include <QGridLayout>
#include <QLabel>
#include <QPainter>
#include <QTimer>

#include <KColorButton>
#include <KComboBox>
#include <KConfigDialog>
#include <KFontRequester>
#include <KGlobalSettings>
#include <KLineEdit>
#include <KIntNumInput>

#include <Plasma/Theme>

CommandWatch::CommandWatch( QObject* p_parent, const QVariantList& p_args )
: Plasma::Applet( p_parent, p_args )
, m_forceUpdate( false )
, m_timer( 0 )
, m_command_edit( 0 )
, m_interval_value( 0 )
, m_interval_combo( 0 )
, m_kfr( 0 )
, m_colorbutton( 0 )
{
  setBackgroundHints( StandardBackground );
  setAspectRatioMode( Plasma::IgnoreAspectRatio );
  setHasConfigurationInterface( true );

  resize( 200, 50 );
}

CommandWatch::~CommandWatch()
{
  KConfigGroup cfg = config();

  cfg.writeEntry( "nextupdate", m_nextUpdate );
  cfg.writeEntry( "output", m_output );
}

void CommandWatch::init()
{
  m_proc.setOutputChannelMode( KProcess::MergedChannels );
  connect( &m_proc, SIGNAL( readyReadStandardOutput() ), SLOT( gotStdOut() ) );

  m_timer = new QTimer( this );
  m_timer->setSingleShot( true );
  connect( m_timer, SIGNAL( timeout() ), SLOT( executeCommand() ) );

  QAction *forceUpdate = new QAction( SmallIcon("system-run"), i18n("Force &Update"), this );
  m_actions.append( forceUpdate );
  connect( forceUpdate, SIGNAL( triggered( bool ) ), SLOT( executeCommand() ) );

  // load configuration, also sets label appearance
  loadConfig();
}

void CommandWatch::loadConfig()
{
  KConfigGroup cfg = config();

  QDateTime current_dt = QDateTime::currentDateTime();

  // load
  m_command = cfg.readEntry( "command" );
  m_timeout = cfg.readEntry( "timeout", 0 );
  m_font = cfg.readEntry( "font", KGlobalSettings::generalFont() );
  m_fontcolor = cfg.readEntry( "fontcolor", Plasma::Theme::defaultTheme()->color( Plasma::Theme::TextColor ) );
  m_nextUpdate = cfg.readEntry( "nextupdate", current_dt );
  m_output = cfg.readEntry( "output" );

  m_proc.setShellCommand( m_command );

  int timeout = m_forceUpdate ? 0 : qMax( 0, current_dt.secsTo( m_nextUpdate ) * 1000 );
  m_timer->start( timeout );
}

void CommandWatch::saveConfig()
{
  KConfigGroup cfg = config();

  int index = m_interval_combo->currentIndex();
  int unit = m_interval_combo->itemData( index ).toInt();
  int timeout = m_interval_value->value() * unit;

  QString command = m_command_edit->text();
  m_forceUpdate = ( command != cfg.readEntry( "command" ) ) || ( timeout != cfg.readEntry( "timeout", 0 ) );
  cfg.writeEntry( "command", command );
  cfg.writeEntry( "timeout", timeout );
  cfg.writeEntry( "font", m_kfr->font() );
  cfg.writeEntry( "fontcolor", m_colorbutton->color() );
}

void CommandWatch::createConfigurationInterface( KConfigDialog *p_parent )
{
  p_parent->setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Apply );
  connect( p_parent, SIGNAL( applyClicked() ), SLOT( saveConfig() ) );
  connect( p_parent, SIGNAL( applyClicked() ), SLOT( loadConfig() ) );
  connect( p_parent, SIGNAL( okClicked() ), SLOT( saveConfig() ) );
  connect( p_parent, SIGNAL( okClicked() ), SLOT( loadConfig() ) );

  QWidget *page = new QWidget();

  QLabel *command_label = new QLabel( i18n("&Command:"), page );
  m_command_edit = new KLineEdit( page );
  m_command_edit->setText( m_command );
  command_label->setBuddy( m_command_edit );

  // command
  QGridLayout *layout = new QGridLayout( page );
  layout->setColumnStretch( 1, 1 );

  // interval
  QHBoxLayout *hbox = new QHBoxLayout();
  QLabel *interval_label = new QLabel( i18n( "&Refresh every:" ), page );
  m_interval_value = new KIntNumInput( page );
  m_interval_value->setMinimum( 0 );
  interval_label->setBuddy( m_interval_value );
  m_interval_combo = new KComboBox( page );

  const int second = 1;
  const int minute = 60;
  const int hour = 3600;
  const int day = 86400;

  m_interval_combo->insertItem( 0, i18n( "seconds"), second );
  m_interval_combo->insertItem( 1, i18n( "minutes"), minute );
  m_interval_combo->insertItem( 2, i18n( "hours" ), hour );
  m_interval_combo->insertItem( 3, i18n( "days" ), day );

  if ( m_timeout > 0 && m_timeout % day == 0 )
  {
    m_interval_value->setValue( m_timeout / day );
    m_interval_combo->setCurrentIndex( 3 );
  }
  else if ( m_timeout > 0 && m_timeout % hour == 0 )
  {
    m_interval_value->setValue( m_timeout / hour );
    m_interval_combo->setCurrentIndex( 2 );
  }
  else if ( m_timeout > 0 && m_timeout % minute == 0 )
  {
    m_interval_value->setValue( m_timeout / minute );
    m_interval_combo->setCurrentIndex( 1 );
  }
  else
  {
    m_interval_value->setValue( m_timeout );
    m_interval_combo->setCurrentIndex( 0 );
  }

  hbox->addWidget( m_interval_value, 1 );
  hbox->addWidget( m_interval_combo );

  // font
  QLabel *font_label = new QLabel( i18n( "&Font:" ), page );
  m_kfr = new KFontRequester( page );
  m_kfr->setFont( m_font );
  font_label->setBuddy( m_kfr );

  // color
  QLabel *font_color_label = new QLabel( i18n( "&Color:" ), page );
  m_colorbutton = new KColorButton( page );
  m_colorbutton->setColor( m_fontcolor );
  font_color_label->setBuddy( m_colorbutton );

  layout->addWidget( command_label, 0, 0 );
  layout->addWidget( m_command_edit, 0, 1 );
  layout->addWidget( interval_label, 1, 0 );
  layout->addLayout( hbox, 1, 1 );
  layout->addWidget( font_label, 2, 0 );
  layout->addWidget( m_kfr, 2, 1 );
  layout->addWidget( font_color_label, 3, 0 );
  layout->addWidget( m_colorbutton, 3, 1 );
  layout->addItem( new QSpacerItem( 0, 0 ), 3, 0 );
  layout->setRowStretch( 3, 1 );

  p_parent->addPage( page, i18nc( "name of the applet", "%1 Settings", name() ), icon() );
}

void CommandWatch::executeCommand()
{
  // don't execute a new instance when something is already being worked on
  if ( m_command.isEmpty() || m_proc.state() != QProcess::NotRunning )
  {
    return;
  }

  m_proc.start();
}

void CommandWatch::gotStdOut()
{
  m_output = QString::fromUtf8( m_proc.readAllStandardOutput() );
  m_nextUpdate = QDateTime::currentDateTime().addSecs( m_timeout );
  if ( m_timeout > 0 )
  {
    m_timer->start( m_timeout * 1000 );
  }

  update();
}

void CommandWatch::paintInterface( QPainter *p_painter, const QStyleOptionGraphicsItem *p_option, const QRect &p_contentsRect )
{
  Q_UNUSED( p_option );
  p_painter->setPen( QPen( m_fontcolor ) );
  p_painter->setFont( m_font );

  p_painter->drawText( p_contentsRect, m_output );
}

QList<QAction *> CommandWatch::contextualActions()
{
  return m_actions;
}

K_EXPORT_PLASMA_APPLET(commandwatch, CommandWatch)
